/**
 * Color class.
 *
 * Declares basic colors in RGB format.
 * Overloaded operator+ lets you "mix" colors, ie: GREEN+BLACK==DARK_GREEN
 * Overloaded operator* lets you brighten colors
 *
 * Written by Robert Osada, March 1999.
 **/
#ifndef __COLOR_H__
#define __COLOR_H__

class Color
{
public:
  static const Color
    WHITE, GRAY, BLACK,
    RED, YELLOW, GREEN, CYAN, BLUE, PURPLE;

  static const Color
    LIGHT_GRAY, LIGHT_RED, LIGHT_YELLOW, LIGHT_GREEN, LIGHT_CYAN, LIGHT_BLUE, LIGHT_PURPLE,
    DARK_GRAY, DARK_RED, DARK_YELLOW, DARK_GREEN, DARK_CYAN, DARK_BLUE, DARK_PURPLE;

  // create a color
  Color (float r_=0, float g_=0, float b_=0) : r(r_), g(g_), b(b_) {}

  // mix colors
  Color  operator +  (const Color &rhs) const { return Color((r+rhs.r)*.5f, (g+rhs.g)*.5f, (b+rhs.b)*.5f); }
  Color& operator += (const Color &rhs)       { r+=rhs.r; g+=rhs.g; b+=rhs.b; return (*this)*=.5f; }

  // scale colors
  Color  operator *  (float s) const { return Color(r*s, g*s, b*s); }
  Color& operator *= (float s)       { r *= s; g *= s; b *= s; return *this; }

  // rgb components
  float r, g, b;
};

#endif
