/**
 * Mesh class.
 * Contains all vertices and faces of a mesh.
 * Parses input file.
 *
 * Written by Robert Osada, September 2000.
 **/
#ifndef __MESH_H__
#define __MESH_H__

#include "Declarations.h"
#include "Face.h"
#include "Vertex.h"

class Mesh
{
public:
  Mesh (FILE *file); // parse mesh

  // counts
  int numV () const { return V.size(); } // number of vertices
  int numF () const { return F.size(); } // number of faces

  // vertex access
  VertexPtr       getVertex (int i)       { return V[i]; }
  const VertexPtr getVertex (int i) const { return V[i]; }

  // face access
  FacePtr       getFace   (int i)       { return F[i]; }
  const FacePtr getFace   (int i) const { return F[i]; }

  // bounding box
  void getBBox (Vector3f& min, Vector3f& max) const;

  // write out mesh
  bool write (FILE *file) const;

  // openGL draw routines
  void drawVertices    () const;
  void drawEdges       () const;
  void drawFaces       () const;
  void drawFacesSmooth () const;

private:
  VECTOR(VertexPtr) V;  // vertices
  VECTOR(FacePtr)   F;  // faces
};

#endif
