/**
 * Misc. definitions
 *
 * STL containers
 * Boolean datatype
 * OpenGL helper routines
 **/
#ifndef __MISC_H__
#define __MISC_H__

#include <assert.h>
#include <iostream.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <GL/glut.h>

// constants
static const float PI = 3.14159265359f;

// 0 parameter functions
inline float Rand () { return (float)rand() / RAND_MAX; }

// unary functions
inline float Abs  (float x) { return x>=0 ? x : -x; }
inline float Sqrt (float x) { return (float)sqrt(x); }

// binary functions
inline float Max  (float x, float y) { return x>=y ? x :  y; }
inline float Min  (float x, float y) { return x<=y ? x :  y; }
inline int   Mod  (int   x, int   y) { int t=x % y; return t>=0 ? t : y+t; }

inline float Max  (float a, float b, float c)          { return Max(Max(a,b),c); }
inline float Max  (float a, float b, float c, float d) { return Max(Max(a,b),Max(c,d)); }
inline float Min  (float a, float b, float c)          { return Min(Min(a,b),c); }
inline float Min  (float a, float b, float c, float d) { return Min(Min(a,b),Min(c,d)); }


/**
 * STL
 **/
#define VECTOR(type) std::vector < type >


/**
 * OpenGL helper routines
 **/
#include "Vector.h"
#include "Color.h"

inline void LoadVertex (const Vector3f &v)
{ glVertex3f(v.x, v.y, v.z); }

inline void LoadNormal (const Vector3f &n)
{ glNormal3f(n.x, n.y, n.z); }

inline void LoadColor (const Color &c)
{ glColor3f(c.r,c.g,c.b); }

#endif

